@@ -16,6 +16,7 @@ from motion.utils import (
     RedisParams,
     UpdateEvent,
     UpdateEventGroup,
+    get_redis_params,
     hash_object,
     loadState,
     logger,
@@ -27,6 +28,7 @@ class Executor:
     def __init__(
         self,
         instance_name: str,
+        cache_ttl: int,
         init_state_func: Optional[Callable],
         init_state_params: Dict[str, Any],
         save_state_func: Optional[Callable],
@@ -36,6 +38,7 @@ class Executor:
         disabled: bool = False,
     ):
         self._instance_name = instance_name
+        self._cache_ttl = cache_ttl
 
         self._init_state_func = init_state_func
         self._init_state_params = init_state_params
@@ -43,13 +46,12 @@ class Executor:
         self._save_state_func = save_state_func
 
         self.running: Any = multiprocessing.Value("b", False)
-        self._redis_con = self._connectToRedis()
+        self._redis_params, self._redis_con = self._connectToRedis()
         try:
             self._redis_con.ping()
         except redis.exceptions.ConnectionError:
-            rp = RedisParams()
             raise ConnectionError(
-                f"Could not connect to a Redis backend {rp}. "
+                f"Could not connect to a Redis backend {self._redis_params}. "
                 + "Please set environment variables MOTION_REDIS_HOST, "
                 + "MOTION_REDIS_PORT, MOTION_REDIS_DB, and/or "
                 + "MOTION_REDIS_PASSWORD to your Redis params."
@@ -95,15 +97,15 @@ class Executor:
         if not disabled:
             self._build_fit_jobs()
 
-    def _connectToRedis(self) -> redis.Redis:
-        rp = RedisParams()
+    def _connectToRedis(self) -> Tuple[RedisParams, redis.Redis]:
+        rp = get_redis_params()
         r = redis.Redis(
             host=rp.host,
             port=rp.port,
             password=rp.password,
             db=rp.db,
         )
-        return r
+        return rp, r
 
     def _loadState(self) -> State:
         return loadState(self._redis_con, self._instance_name, self._load_state_func)
@@ -119,69 +121,73 @@ class Executor:
         return {}
 
     def _build_fit_jobs(self) -> None:
-        """Builds update jobs."""
-        # Set up worker loops
-        self.worker_tasks = {}
-        # self.worker_stop_events = {}
-
-        rp = RedisParams()
+        """Builds update job."""
         # self.worker_states = {}
 
+        # Set up update task
+        self.route_dict_for_fit = {}
+        self.channel_dict_for_fit = {}
+        self.queue_ids_for_fit = []
         for rkey, routes in self._update_routes.items():
             for udf_name, route in routes.items():
-                pname = f"{self._instance_name}::{rkey}::{udf_name}"
-                multiprocessing.Event()
-                self.worker_tasks[pname] = UpdateTask(
-                    self._instance_name,
-                    route,
-                    save_state_func=self._save_state_func,
-                    load_state_func=self._load_state_func,
-                    queue_identifier=self._get_queue_identifier(rkey, udf_name),
-                    channel_identifier=self._get_channel_identifier(rkey, udf_name),
-                    redis_host=rp.host,
-                    redis_port=rp.port,
-                    redis_db=rp.db,
-                    redis_password=rp.password,  # type: ignore
-                    running=self.running,
+                queue_id = self._get_queue_identifier(rkey, udf_name)
+                self.queue_ids_for_fit.append(queue_id)
+                self.route_dict_for_fit[queue_id] = route
+                self.channel_dict_for_fit[queue_id] = self._get_channel_identifier(
+                    rkey, udf_name
                 )
-                # self.worker_stop_events[pname] = worker_stop_event
-                self.worker_tasks[pname].start()
+
+        self.worker_task = None
+        if self.queue_ids_for_fit:
+            self.worker_task = UpdateTask(
+                self._instance_name,
+                routes=self.route_dict_for_fit,
+                save_state_func=self._save_state_func,
+                load_state_func=self._load_state_func,
+                queue_identifiers=self.queue_ids_for_fit,
+                channel_identifiers=self.channel_dict_for_fit,
+                redis_host=self._redis_params.host,
+                redis_port=self._redis_params.port,
+                redis_db=self._redis_params.db,
+                redis_password=self._redis_params.password,  # type: ignore
+                running=self.running,
+            )
+            self.worker_task.start()
 
         # Set up a monitor thread
         self.stop_event = threading.Event()
         self.monitor_thread = threading.Thread(
-            target=self._monitor_processes, daemon=True
+            target=self._monitor_process, daemon=True
         )
         self.monitor_thread.start()
 
-    def _monitor_processes(self) -> None:
-        rp = RedisParams()
+    def _monitor_process(self) -> None:
+        if not self.worker_task:
+            return
+
         while not self.stop_event.is_set():
-            # Loop through processes to see if they are alive
-            for pname, process in self.worker_tasks.items():
-                if not process.is_alive():
-                    logger.debug(
-                        f"Failed to detect heartbeat for update task {pname}."
-                        + " Restarting the task in the background."
-                    )
-                    # Restart
-                    rkey = pname.split("::")[1]
-                    udf_name = pname.split("::")[2]
-                    route = self._update_routes[rkey][udf_name]
-                    self.worker_tasks[pname] = UpdateTask(
-                        self._instance_name,
-                        route,
-                        save_state_func=self._save_state_func,
-                        load_state_func=self._load_state_func,
-                        queue_identifier=self._get_queue_identifier(rkey, udf_name),
-                        channel_identifier=self._get_channel_identifier(rkey, udf_name),
-                        redis_host=rp.host,
-                        redis_port=rp.port,
-                        redis_db=rp.db,
-                        redis_password=rp.password,  # type: ignore
-                        running=self.running,
-                    )
-                    self.worker_tasks[pname].start()
+            # See if the update task is alive
+            if not self.worker_task.is_alive():
+                logger.debug(
+                    f"Failed to detect heartbeat for {self.worker_task.name}."
+                    + " Restarting the task in the background."
+                )
+
+                # Restart
+                self.worker_task = UpdateTask(
+                    self._instance_name,
+                    routes=self.route_dict_for_fit,
+                    save_state_func=self._save_state_func,
+                    load_state_func=self._load_state_func,
+                    queue_identifiers=self.queue_ids_for_fit,
+                    channel_identifiers=self.channel_dict_for_fit,
+                    redis_host=self._redis_params.host,
+                    redis_port=self._redis_params.port,
+                    redis_db=self._redis_params.db,
+                    redis_password=self._redis_params.password,  # type: ignore
+                    running=self.running,
+                )
+                self.worker_task.start()
 
             if self.stop_event.is_set():
                 break
@@ -211,20 +217,11 @@ class Executor:
         self.stop_event.set()
         self.running.value = False
 
-        processes_to_wait_for = []
-        for process in self.worker_tasks.values():
-            if psutil.pid_exists(process.pid):
-                # os.kill(process.pid, signal.SIGUSR1)  # type:ignore
-                # Set stop event
-                # self.worker_stop_events[pname].set()
-                processes_to_wait_for.append(process)
+        if self.worker_task and psutil.pid_exists(self.worker_task.pid):
+            self.worker_task.join()
 
         self._redis_con.close()
 
-        # Join update processes
-        for process in processes_to_wait_for:
-            process.join()
-
         self.monitor_thread.join()
 
     def _updateState(self, new_state: Dict[str, Any]) -> None:
@@ -331,6 +328,10 @@ class Executor:
                 )
             self.version = int(v)
 
+        # If caching is disabled, return
+        if self._cache_ttl == 0:
+            return route_run, serve_result, props, None
+
         # Try hashing the value
         try:
             value_hash = hash_object(props)
@@ -352,7 +353,6 @@ class Executor:
         self,
         key: str,
         props: Dict[str, Any],
-        cache_ttl: int,
         ignore_cache: bool,
         force_refresh: bool,
         flush_update: bool,
@@ -394,7 +394,7 @@ class Executor:
                     self._redis_con.set(
                         cache_result_key,
                         cloudpickle.dumps(props),
-                        ex=cache_ttl,
+                        ex=self._cache_ttl,
                     )
 
         # Run the update routes
@@ -412,7 +412,6 @@ class Executor:
         self,
         key: str,
         props: Dict[str, Any],
-        cache_ttl: int,
         ignore_cache: bool,
         force_refresh: bool,
         flush_update: bool,
@@ -454,7 +453,7 @@ class Executor:
                     self._redis_con.set(
                         cache_result_key,
                         cloudpickle.dumps(props),
-                        ex=cache_ttl,
+                        ex=self._cache_ttl,
                     )
 
         # Run the update routes
